-- This file is part of VulkAda.

-- VulkAda is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as
-- published by the Free Software Foundation, either version 3 of
-- the License, or (at your option) any later version.

-- VulkAda is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.

-- You should have received a copy of the GNU Lesser General Public
-- License along with VulkAda.
-- If not, see <http://www.gnu.org/licenses/>.

-- Copyright 2024 Phaser Cat Games LLC

-- Event related subprograms

with Vulkan.Exceptions;

package body Vulkan.Events is
    function Create(Device: in Vulkan.Device;
                    Allocator: aliased in Allocation_Callbacks;
                    Event: out Vulkan.Event) return Result is
        Create_Info: C.Event_Create_Info_C;
    begin
        return C.vkCreateEvent(Device,
                               Create_Info,
                               Allocator'Access,
                               Event);
    end Create;

    function Create(Device: in Vulkan.Device;
                    Allocator: aliased in Allocation_Callbacks)
                        return Event is
        E: Event;
    begin
        Exceptions.Check(Create(Device, Allocator, E));

        return E;
    end Create;

    function Create(Device: in Vulkan.Device;
                    Event: out Vulkan.Event) return Result is
        Create_Info: C.Event_Create_Info_C;
    begin
        return C.vkCreateEvent(Device,
                               Create_Info,
                               null,
                               Event);
    end Create;

    function Create(Device: in Vulkan.Device) return Event is
        E: Event;
    begin
        Exceptions.Check(Create(Device, E));

        return E;
    end Create;

    procedure Destroy(Device: in Vulkan.Device;
                      Event: in out Vulkan.Event;
                      Allocator: aliased in Allocation_Callbacks) is
    begin
        C.vkDestroyEvent(Device, Event, Allocator'Access);
        Event := No_Event;
    end Destroy;

    procedure Destroy(Device: in Vulkan.Device;
                      Event: in out Vulkan.Event) is
    begin
        C.vkDestroyEvent(Device, Event, null);
        Event := No_Event;
    end Destroy;

    procedure Set(Device: in Vulkan.Device;
                  Event: in Vulkan.Event) is
    begin
        Exceptions.Check(Set(Device, Event));
    end Set;
    
    procedure Reset(Device: in Vulkan.Device;
                    Event: in Vulkan.Event) is
    begin
        Exceptions.Check(Reset(Device, Event));
    end Reset;
end Vulkan.Events;

